"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.connectServer = exports.inspectServers = exports.ensureArray = exports.parseAddress = exports.agent = void 0;

var _stream = require("stream");

var _http = _interopRequireDefault(require("http"));

var _parameter = _interopRequireDefault(require("./parameter"));

var _settings = _interopRequireDefault(require("./settings"));

var _debug = _interopRequireDefault(require("debug"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var agent = new _http["default"].Agent({
  maxSockets: 0,
  keepAlive: false,
  timeout: 0
});
exports.agent = agent;

var parseAddress = function parseAddress(commands, environment) {
  return function (srvr) {
    if (typeof srvr !== 'string') {
      return null;
    }

    var hostWithPort = srvr.match(/^\[?([^\]]+)\]?:(\d+)$/);
    var serverOptions = {
      hostname: srvr,
      port: _settings["default"].port,
      path: '/',
      method: 'POST',
      headers: {
        'Transfer-Encoding': 'chunked',
        'Content-Type': 'application/json'
      },
      agent: agent
    };
    commands.filter(Boolean).forEach(function (command, index) {
      serverOptions.headers["X-Command-".concat(index)] = _parameter["default"].pack(command);
    });
    Object.keys(environment).filter(function (keyEnv) {
      return environment[keyEnv];
    }).forEach(function (keyEnv, index) {
      serverOptions.headers["X-Environment-".concat(index)] = _parameter["default"].pack({
        k: keyEnv,
        v: environment[keyEnv]
      });
    });

    if (hostWithPort) {
      return _objectSpread({}, serverOptions, {
        hostname: hostWithPort[1],
        port: Number(hostWithPort[2])
      });
    }

    return serverOptions;
  };
};

exports.parseAddress = parseAddress;

var ensureArray = function ensureArray(a) {
  return Array.isArray(a) ? a : [a];
};

exports.ensureArray = ensureArray;

var inspectServers = function inspectServers(servers, commands, environment, ns) {
  return ensureArray(servers).filter(Boolean).filter(function (elem, pos, arr) {
    return arr.indexOf(elem) === pos;
  }).map(parseAddress(commands, environment)).map(function (s) {
    return Array(ns || _settings["default"].nShards).fill(s);
  }) // multiple each line
  .reduce(function (a, b) {
    return a.concat(b);
  }, []);
}; // flatten all


exports.inspectServers = inspectServers;

var connectServer = function connectServer(ezs) {
  return function (serverOptions, index) {
    var hostname = serverOptions.hostname,
        port = serverOptions.port;
    var connected = false;
    serverOptions.headers = _objectSpread({}, ezs.encodingMode(), {}, serverOptions.headers);
    var input = new _stream.PassThrough(ezs.objectMode());
    var output = new _stream.PassThrough(ezs.objectMode());

    var handle = _http["default"].request(serverOptions, function (res) {
      connected = true;
      (0, _debug["default"])('ezs')("http://".concat(hostname, ":").concat(port, " send code ").concat(res.statusCode));

      if (res.statusCode === 200) {
        res.pipe(ezs.uncompress(res.headers)).pipe(ezs('unpack')).pipe(ezs('ungroup')).pipe(output);
        return 1;
      }

      if (res.statusCode === 500) {
        var errmsg = _parameter["default"].decode(res.headers['x-error']);

        output.write(new Error("Server sent: ".concat(errmsg)));
        output.end();
        return 2;
      }

      output.write(new Error("http://".concat(hostname, ":").concat(port, " at item #").concat(index, " return ").concat(res.statusCode)));
      return 3;
    });

    handle.on('error', function (e) {
      handle.abort();

      if (!connected) {
        output.write(new Error("http://".concat(hostname || '?', ":").concat(port || '?', " at item #").concat(index, " return ").concat(e.message)));
        return output.end();
      }

      (0, _debug["default"])('ezs')("http://".concat(hostname, ":").concat(port, " was stopped properly following ").concat(e));
      return 4;
    });
    handle.setNoDelay(false);
    input.pipe(ezs('group')).pipe(ezs('pack')).pipe(ezs.compress(ezs.encodingMode())).pipe(handle);
    var duplex = [input, output];
    return duplex;
  };
};

exports.connectServer = connectServer;