"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.compressStream = compressStream;
exports.uncompressStream = uncompressStream;

var _stream = require("stream");

var _debug = _interopRequireDefault(require("debug"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var resolve = function resolve(name) {
  try {
    require.resolve(name);

    return true;
  } catch (e) {
    return false;
  }
};

var chooseZ = function chooseZ() {
  if (resolve('zlib')) {
    // eslint-disable-next-line
    return require('zlib');
  }

  return null;
};

var z = chooseZ();

function compressStream(ezs) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var encoding = opts['Content-Encoding'] || opts['content-encoding'] || 'identity';

  if (typeof z.createGunzip === 'function' && encoding === 'gzip') {
    (0, _debug["default"])('ezs')('ezs will use zlib to compress stream.');
    return z.createGzip();
  }

  (0, _debug["default"])('ezs')('ezs will not compress stream.');
  return new _stream.PassThrough(ezs.bytesMode());
}

function uncompressStream(ezs) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var encoding = opts['Content-Encoding'] || opts['content-encoding'] || 'identity';

  if (typeof z.createGunzip === 'function' && encoding === 'gzip') {
    (0, _debug["default"])('ezs')('ezs will use zlib to uncompress stream.');
    return z.createGunzip();
  }

  (0, _debug["default"])('ezs')('ezs will not uncompress stream.');
  return new _stream.PassThrough(ezs.bytesMode());
}