"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _debug = _interopRequireDefault(require("debug"));

var _feed = _interopRequireDefault(require("./feed"));

var _shell = _interopRequireDefault(require("./shell"));

var _concurrentQueue = _interopRequireDefault(require("concurrent-queue"));

var _SafeTransform2 = _interopRequireDefault(require("./SafeTransform"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function createErrorWith() {
  var error = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  var funcName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
  var stk = String(error.stack).split('\n');
  var erm = stk.shift();
  var fn = funcName ? " in [".concat(funcName, "]") : '';
  var msg = "(item #".concat(index).concat(fn, " failed with ").concat(erm, ")\n\t").concat(stk.slice(0, 10).join('\n\t'));
  var err = Error(msg);
  Error.captureStackTrace(err, createErrorWith);
  (0, _debug["default"])('ezs')('Caught an', err);
  return err;
}

var Engine =
/*#__PURE__*/
function (_SafeTransform) {
  _inherits(Engine, _SafeTransform);

  function Engine(ezs, func, params, environment) {
    var _this;

    _classCallCheck(this, Engine);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(Engine).call(this, ezs.objectMode()));
    _this.func = func;
    _this.funcName = String(func.name);
    _this.index = 0;
    _this.params = params || {};
    _this.scope = {};
    _this.ezs = ezs;
    _this.environment = environment || {};
    _this.nullWasSent = false;
    _this.queue = (0, _concurrentQueue["default"])().limit(ezs.settings.queue).process(function (task, cb) {
      _this.execWith(task, cb);
    });

    _this.on('pipe', function (src) {
      _this.parentStream = src;
    });

    return _this;
  }

  _createClass(Engine, [{
    key: "_transform",
    value: function _transform(chunk, encoding, done) {
      if (this.nullWasSent) {
        if (this.parentStream && this.parentStream.unpipe) {
          this.parentStream.unpipe(this);
        }

        return done();
      }

      this.index += 1;

      if (chunk instanceof Error) {
        this.push(chunk);
        return done();
      }

      return this.queue(chunk, done);
    }
  }, {
    key: "_flush",
    value: function _flush(done) {
      if (this.nullWasSent) {
        return done();
      }

      this.index += 1;
      return this.queue(null, done);
    }
  }, {
    key: "execWith",
    value: function execWith(chunk, done) {
      var _this2 = this;

      var currentIndex = this.index;

      if (chunk === null && currentIndex === 1) {
        this.nullWasSent = true;
        this.push(null);
        return done();
      }

      var warn = function warn(error) {
        _this2.emit('error', createErrorWith(error, currentIndex, _this2.funcName));
      };

      var push = function push(data) {
        if (data instanceof Error) {
          (0, _debug["default"])('ezs')("Ignoring error at item #".concat(currentIndex));
          return _this2.push(createErrorWith(data, currentIndex, _this2.funcName));
        }

        if (data === null) {
          _this2.nullWasSent = true;
        }

        return _this2.push(data);
      };

      var feed = new _feed["default"](push, done, warn);

      try {
        this.scope.isFirst = function () {
          return currentIndex === 1;
        };

        this.scope.getIndex = function () {
          return currentIndex;
        };

        this.scope.isLast = function () {
          return chunk === null;
        };

        this.scope.getEnv = function (name) {
          return name === undefined ? _this2.environment : _this2.environment[name];
        };

        this.scope.ezs = this.ezs;

        this.scope.emit = function (d, c) {
          return _this2.emit(d, c);
        };

        this.scope.getParams = function () {
          return _this2.params;
        };

        this.scope.getParam = function (name, defval) {
          if (_this2.params[name] !== undefined) {
            return (0, _shell["default"])(_this2.params[name], chunk, _this2.environment);
          }

          return defval;
        };

        Promise.resolve(this.func.call(this.scope, chunk, feed, currentIndex))["catch"](function (e) {
          (0, _debug["default"])('ezs')("Async error thrown at item #".concat(currentIndex, ", pipeline is broken"));

          _this2.emit('error', createErrorWith(e, currentIndex, _this2.funcName));

          done();
        });
      } catch (e) {
        (0, _debug["default"])('ezs')("Sync error thrown at item #".concat(currentIndex, ", pipeline carries errors"));
        this.push(createErrorWith(e, currentIndex, this.funcName));
        done();
      }
    }
  }]);

  return Engine;
}(_SafeTransform2["default"]);

exports["default"] = Engine;