"use strict";

var _stream = require("stream");

var _streamWrite = _interopRequireDefault(require("stream-write"));

var _engine = _interopRequireDefault(require("./engine"));

var _script = _interopRequireDefault(require("./script"));

var _file = _interopRequireDefault(require("./file"));

var _output = _interopRequireDefault(require("./output"));

var _commands = _interopRequireDefault(require("./commands"));

var _catcher = _interopRequireDefault(require("./catcher"));

var _statements = _interopRequireDefault(require("./statements"));

var _statement = _interopRequireDefault(require("./statement"));

var _meta = _interopRequireDefault(require("./meta"));

var _server = _interopRequireDefault(require("./server"));

var _settings = _interopRequireDefault(require("./settings"));

var _compactor = require("./compactor");

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var ezs = function ezs(name, options, environment) {
  return new _engine["default"](ezs, _statement["default"].get(ezs, name, options), options, environment);
};

var ezsPath = [process.cwd()];
ezs.settings = _settings["default"];

ezs.objectMode = function () {
  return {
    objectMode: true,
    highWaterMark: _settings["default"].highWaterMark.object
  };
};

ezs.bytesMode = function () {
  return {
    objectMode: false,
    highWaterMark: ezs.settings.highWaterMark.bytes
  };
};

ezs.encodingMode = function () {
  return {
    'Content-Encoding': _settings["default"].encoding
  };
};

ezs.metaString = function (commands, options) {
  return new _meta["default"](ezs, commands, options);
};

ezs.metaFile = function (filename, options) {
  return new _meta["default"](ezs, (0, _file["default"])(ezs, filename), options);
};

ezs.parseString = function (commands) {
  return (0, _script["default"])(commands);
};

ezs.parseFile = function (filename) {
  return (0, _script["default"])((0, _file["default"])(ezs, filename));
};

ezs["catch"] = function (func) {
  return new _catcher["default"](func);
};

ezs.toBuffer = function (options) {
  return new _output["default"](options);
};

ezs.use = function (plugin) {
  return _statement["default"].set(ezs, plugin);
};

ezs.addPath = function (p) {
  return ezsPath.push(p);
};

ezs.getPath = function () {
  return ezsPath;
};

ezs.loadScript = function (file) {
  return (0, _file["default"])(ezs, file);
};

ezs.compileScript = function (script) {
  return new _commands["default"](ezs.parseString(script));
};

ezs.createCommand = function (command, environment) {
  var mode = command.mode || _constants.M_NORMAL;

  if (!command.name) {
    throw new Error("Bad command : ".concat(command.name));
  }

  if (command.use) {
    _statement["default"].load(ezs, command.use);
  }

  if (mode === _constants.M_NORMAL || mode === _constants.M_DISPATCH) {
    return ezs(command.name, command.args, environment);
  }

  if (mode === _constants.M_SINGLE || mode === 'single'
  /* Backward compatibility */
  ) {
      return ezs('singleton', _objectSpread({}, command.args, {
        statement: command.name
      }), environment);
    }

  throw new Error("Bad mode: ".concat(mode));
};

ezs.compileCommands = function (commands, environment) {
  if (!Array.isArray(commands)) {
    throw new Error('Pipeline works with an array of commands.');
  }

  var cmds = _toConsumableArray(commands);

  cmds.push({
    mode: _constants.M_NORMAL,
    name: 'transit',
    args: {}
  });
  var streams = cmds.map(function (command) {
    return ezs.createCommand(command, environment);
  });

  if (streams.length === 1) {
    return new _stream.PassThrough(ezs.objectMode());
  }

  return streams;
};

ezs.writeTo = _streamWrite["default"];

ezs.createPipeline = function (input, streams) {
  return streams.reduce(function (amont, aval) {
    return amont.pipe(aval);
  }, input);
};

ezs.compress = function (options) {
  return (0, _compactor.compressStream)(ezs, options);
};

ezs.uncompress = function (options) {
  return (0, _compactor.uncompressStream)(ezs, options);
};

ezs.createStream = function (options) {
  return new _stream.PassThrough(options);
};

ezs.createServer = function (port, path) {
  return _server["default"].createServer(ezs, port, path);
};

ezs.createCluster = function (port, path) {
  return _server["default"].createCluster(ezs, port, path);
};

ezs.use(_statements["default"]);
module.exports = ezs;