"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = Script;

var _expression = _interopRequireDefault(require("./expression"));

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var regex = {
  section: /^\s*\[\s*([^\]]*)\s*\]\s*$/,
  param: /^\s*([\w.\-_]+)\s*[=: ]\s*(.*?)\s*$/,
  comment: /^\s*[;#].*$/
};

var parseOpts = function parseOpts(obj) {
  var res = {};
  Object.keys(obj).forEach(function (key) {
    var val = obj[key].length === 1 ? obj[key][0] : obj[key];
    res[key] = new _expression["default"](val);
  });
  return res;
};

function Script(commands) {
  if (!commands) {
    return [];
  }

  var lines = commands.split(/\r\n|\r|\n/);
  var result = [];
  lines.forEach(function (line) {
    if (!regex.comment.test(line)) {
      if (regex.param.test(line)) {
        var match = line.match(regex.param);
        var paramName = match[1];
        var paramValue = match[2];
        var section = result.length > 0 ? result[result.length - 1] : null;

        if (section) {
          if (!section.args[paramName]) {
            result[result.length - 1].args[paramName] = [];
          }

          result[result.length - 1].args[paramName].push(paramValue);
        }
      } else if (regex.section.test(line)) {
        var matches0 = line.match(regex.section);
        var matches1 = matches0[1].match(/([:\w]+)\?(\w+)/);
        var mode = _constants.M_NORMAL;
        var name = 'debug';
        var use = '';
        var test = '';

        if (Array.isArray(matches1)) {
          var _matches = _slicedToArray(matches1, 3);

          name = _matches[1];
          mode = _matches[2];
        } else {
          mode = _constants.M_NORMAL;

          var _matches2 = _slicedToArray(matches0, 2);

          name = _matches2[1];
        }

        if (name.indexOf(':') !== -1) {
          var _name$split = name.split(':');

          var _name$split2 = _slicedToArray(_name$split, 2);

          use = _name$split2[0];
          name = _name$split2[1];
        }

        var newSection = {
          mode: mode,
          name: name,
          test: test,
          args: {},
          use: use
        };
        result.push(newSection);
      }
    }
  });
  var cmd = result.map(function (command) {
    return _objectSpread({}, command, {
      args: parseOpts(command.args)
    });
  });
  return cmd;
}