"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _cluster = _interopRequireDefault(require("cluster"));

var _http = _interopRequireDefault(require("http"));

var _httpShutdown = _interopRequireDefault(require("http-shutdown"));

var _filenameRegex = _interopRequireDefault(require("filename-regex"));

var _url = require("url");

var _debug = _interopRequireDefault(require("debug"));

var _knownPipeline = _interopRequireDefault(require("./knownPipeline"));

var _unknownPipeline = _interopRequireDefault(require("./unknownPipeline"));

var _serverInformation = _interopRequireDefault(require("./serverInformation"));

var _errorHandler = _interopRequireDefault(require("./errorHandler"));

var _settings = _interopRequireDefault(require("../settings"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var isPipeline = function isPipeline(filename) {
  var f = filename.match((0, _filenameRegex["default"])());
  return f && f.shift() !== undefined;
};

function createMidddleware(ezs, path, method, pathname) {
  if (path === false && method === 'POST' && pathname === '/') {
    (0, _debug["default"])('ezs')("Create middleware 'unknownPipeline' for ".concat(method, " ").concat(pathname));
    return (0, _unknownPipeline["default"])(ezs);
  }

  if (method === 'GET' && pathname === '/') {
    (0, _debug["default"])('ezs')("Create middleware 'serverInformation' for ".concat(method, " ").concat(pathname));
    return (0, _serverInformation["default"])(ezs, path);
  }

  if (path !== false && isPipeline(pathname)) {
    (0, _debug["default"])('ezs')("Create middleware 'knownPipeline' for ".concat(method, " ").concat(pathname));
    return (0, _knownPipeline["default"])(ezs, path);
  }

  var error = new Error("Unable to create middleware for ".concat(method, " ").concat(pathname));
  return function (request, response) {
    return (0, _errorHandler["default"])(request, response)(error, 404);
  };
}

var signals = ['SIGINT', 'SIGTERM'];

function createServer(ezs, serverPort, serverPath) {
  var server = (0, _httpShutdown["default"])(_http["default"].createServer(function (request, response) {
    var method = request.method;
    response.socket.setNoDelay(false);
    request.url = (0, _url.parse)(request.url, true);
    var middleware = createMidddleware(ezs, serverPath, method, request.url.pathname);

    try {
      middleware(request, response);
    } catch (error) {
      (0, _errorHandler["default"])(request, response)(error);
    }

    return true;
  }));
  server.setTimeout(0);
  server.listen(serverPort);
  signals.forEach(function (signal) {
    return process.on(signal, function () {
      (0, _debug["default"])('ezs')("Signal received, stoping server with PID ".concat(process.pid));
      server.shutdown(function () {
        return process.exit(0);
      });
    });
  });
  (0, _debug["default"])('ezs')("Server starting with PID ".concat(process.pid, " and listening on port ").concat(serverPort));
  return server;
}

function createCluster(ezs, serverPort, serverPath) {
  var term = false;

  if (_cluster["default"].isMaster) {
    for (var i = 0; i < _settings["default"].nShards; i += 1) {
      _cluster["default"].fork();
    }

    _cluster["default"].on('exit', function () {
      if (!term) {
        _cluster["default"].fork();
      }
    });

    signals.forEach(function (signal) {
      process.on(signal, function () {
        term = true;
        Object.keys(_cluster["default"].workers).forEach(function (id) {
          return _cluster["default"].workers[id].kill();
        });
      });
    });
  } else {
    createServer(ezs, serverPort, serverPath);
  }

  return _cluster["default"];
}

var _default = {
  createServer: createServer,
  createCluster: createCluster
};
exports["default"] = _default;