"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

var _streamIterate = _interopRequireDefault(require("stream-iterate"));

var _path = require("path");

var _stream = require("stream");

var _debug = _interopRequireDefault(require("debug"));

var _errorHandler = _interopRequireDefault(require("./errorHandler"));

var _file = require("../file");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var dispositionFrom = function dispositionFrom(_ref) {
  var extension = _ref.extension;
  return extension ? "dump.".concat(extension) : extension;
};

var encodingFrom = function encodingFrom(headers) {
  return headers && headers['content-encoding'] ? headers['content-encoding'] : false;
};

var typeFrom = function typeFrom(_ref2) {
  var mimeType = _ref2.mimeType;
  return mimeType;
};

var knownPipeline = function knownPipeline(ezs, serverPath) {
  return function (request, response) {
    var headers = request.headers;
    var triggerError = (0, _errorHandler["default"])(request, response);
    var _request$url = request.url,
        pathname = _request$url.pathname,
        query = _request$url.query;
    var files = pathname.slice(1).split(',').map(function (file) {
      return (0, _path.join)(serverPath, file);
    }).filter(function (file) {
      return (0, _file.isFile)(file);
    });
    var meta = files.map(function (file) {
      return ezs.metaFile(file);
    }).reduce(function (prev, cur) {
      return _lodash["default"].merge(cur, prev);
    }, {});

    if (files.length === 0) {
      triggerError(new Error("Cannot find ".concat(pathname)), 404);
      return false;
    }

    response.setHeader('Content-Encoding', encodingFrom(headers) || 'identity');
    response.setHeader('Content-Disposition', dispositionFrom(meta) || 'inline');
    response.setHeader('Content-Type', typeFrom(meta) || 'application/octet-stream');
    (0, _debug["default"])('ezs')("PID ".concat(process.pid, " will execute ").concat(pathname, " commands with ").concat(Object.keys(query).length || 0, " global parameters"));
    var read = (0, _streamIterate["default"])(request);
    var input = new _stream.PassThrough();

    var createInput = function createInput(firstChunk) {
      var midput;

      if (!firstChunk) {
        midput = input;
        input.write(pathname);
      } else {
        midput = input.pipe(ezs('truncate', {
          length: headers['content-length']
        })).pipe(ezs.uncompress(headers)).on('error', triggerError);
        input.write(firstChunk);
      }

      return midput;
    };

    var loop = function loop(midput) {
      return read(function (err, data, next) {
        if (err) {
          triggerError(err);
          return false;
        }

        next();

        if (data === null) {
          midput.end();
          return null;
        }

        if (data) {
          midput.write(data);
          loop(midput);
          return true;
        }

        return false;
      });
    };

    return read(function (firstError, firstChunk, firstCalled) {
      if (firstError) {
        triggerError(firstError);
        return false;
      }

      var inputBis = createInput(firstChunk);
      ezs.createPipeline(inputBis, files.map(function (file) {
        return ezs('delegate', {
          file: file
        }, query);
      })).pipe(ezs["catch"](function (e) {
        return e;
      })).on('error', triggerError).pipe(ezs(function (data, feed) {
        if (!response.headersSent) {
          response.writeHead(200);
        }

        return feed.send(data);
      })).pipe(ezs.toBuffer()).pipe(ezs.compress(headers)).pipe(response);
      firstCalled();

      if (firstChunk) {
        return loop(input);
      }

      return input.end();
    });
  };
};

var _default = knownPipeline;
exports["default"] = _default;