"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _debug = _interopRequireDefault(require("debug"));

var _file = require("./file");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var pluginsList = {};

function all() {
  return Object.keys(pluginsList);
}

function load(ezs, name) {
  var before1 = Object.keys(pluginsList);
  var fileName = (0, _file.useFile)(ezs, name);

  if (!fileName) {
    throw new Error("'".concat(name, "' is not loaded. It was not found (try to install it)."));
  } // eslint-disable-next-line


  ezs.use(require(fileName));
  var after1 = Object.keys(pluginsList);
  var diff1 = after1.filter(function (item) {
    return before1.indexOf(item) === -1;
  });

  if (diff1.length > 0) {
    (0, _debug["default"])('ezs')("These statements are registered: ".concat(diff1.join(',')));
  }
}

function get(ezs, plugin, opts) {
  if (plugin === 'use') {
    // very special case : to load statement in the current pipeline
    var args = !Array.isArray(opts) ? [opts] : opts;
    args.forEach(function (arg) {
      if (arg.plugin) {
        var names = arg.plugin.get();

        if (!Array.isArray(names)) {
          names = [names];
        }

        names.forEach(function (name) {
          return load(ezs, name);
        });
      }
    });
    return function (data, feed) {
      return feed.send(data);
    };
  }

  if (typeof plugin === 'function') {
    return plugin;
  }

  if (typeof plugin === 'string' && pluginsList[plugin]) {
    return pluginsList[plugin];
  }

  if (_typeof(plugin) === 'object') {
    var firstKey = Object.keys(plugin).slice(0, 1);

    if (typeof plugin[firstKey] === 'function') {
      return plugin[firstKey];
    }
  }

  throw new Error("'".concat(plugin, "' is not loaded. It's not a valid statement function."));
}

function set(ezs, plugin) {
  if (_typeof(plugin) !== 'object') {
    throw new Error("Statement is not loaded. It's not a valid plugin (should be an object).");
  }

  var pluginList = plugin["default"] ? plugin["default"] : plugin; // ES6 hack

  Object.keys(pluginList).forEach(function (pluginName) {
    if (typeof pluginList[pluginName] === 'function') {
      pluginsList[pluginName] = pluginList[pluginName];
    } else if (_typeof(pluginList[pluginName]) === 'object') {
      ezs.use(pluginList[pluginName]);
    } else {
      throw new Error("".concat(pluginName, " is not loaded. It's not a valid plugin."));
    }
  });
  return ezs;
}

function exists(ezs, pluginName) {
  return pluginsList[pluginName] !== undefined;
}

var _default = {
  get: get,
  set: set,
  all: all,
  load: load,
  exists: exists
};
exports["default"] = _default;