"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = debug;

var _util = _interopRequireDefault(require("util"));

var _lodash = _interopRequireDefault(require("lodash"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/**
 * Take `Object` , print it and throw the same object
 *
 * @param {String} [level=log] console level : log or error
 * @param {String} [text=valueOf] text before the dump
 * @param {String} [path] path of field to print
 * @returns {Object}
 */
function debug(data, feed) {
  if (this.isLast()) {
    return feed.send(data);
  }

  var level = this.getParam('level', 'error');
  var text = this.getParam('text', 'valueOf');
  var path = this.getParam('path', []);
  var keys = Array.isArray(path) ? path : [path];
  var output = keys.length === 0 ? data : _lodash["default"].pick(data, keys); // eslint-disable-next-line

  if (typeof console[level] === 'function') {
    var logOpts = {
      showHidden: false,
      depth: 3,
      colors: true
    }; // eslint-disable-next-line

    var logFunc = console[level];
    logFunc(text.concat('#').concat(this.getIndex()).concat(' ->'), _util["default"].inspect(output, logOpts));
  }

  return feed.send(data);
}